/*	This program is released under QT Public License.
 *  for details please refer to the file LICENSE
 *   
 *  Disclaimer: the program comes "as is" and with absolutely no warranty 
 *  
 *  © 2010 TCM
 */


package converter;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Observable;

import javax.imageio.ImageIO;


public class ToBinIO extends Observable {
	
	private int x,y,k;
	private BufferedImage bi;  
	
	private byte[] buffer;
	
	
	public void toBin(File in,String outputDir) {
		try {
			updateGUI("Parsing "+in.getName());
			bi=ImageIO.read(in);
			if (bi==null) {
				updateGUI("Error: PNG file is invalid" );
				return;
			}
			
			x=0;
			y=0;
			buffer=new byte[4096];
			
			int i1=getInt();
			nextPixel();
			int i2=getInt();
			nextPixel();
			int i3=getInt();
			nextPixel();
			int i4=getInt();
			nextPixel();
			long len=intToLong(i1, i2, i3, i4);
			
			i1=getInt();
			nextPixel();
			i2=getInt();
			nextPixel();
			i3=getInt();
			nextPixel();
			i4=getInt();
			nextPixel();
			long pos=intToLong(i1, i2, i3, i4);
			
			StringBuilder sb=new StringBuilder();
			
			int fileNameLen=getInt();
			nextPixel();
			
			//boolean b=false;
			
			int rgb,i=0;
			while (true) {
				
				rgb=getRGB();
				
				if (i%3==0) {
					sb.append((char)getRed(rgb));
					i++;
					if (i==fileNameLen) {
						nextPixel();
						break;
					}
					//b=true;
				} else if (i%3==1) {
					sb.append((char)getGreen(rgb));
					i++;
					if (i==fileNameLen) {
						nextPixel();
						break;
					}
					//b=true;
				} else {
					sb.append((char)getBlue(rgb));
					nextPixel();
					i++;
					if (i==fileNameLen) {
						break;
					}
				}
				
				
			}
			
			
			File out=new File(outputDir+sb.toString());
			
//			if (!out.getParentFile().exists()) {
//				out.mkdirs();
//			}
			if (!out.exists()) out.createNewFile();
			
			RandomAccessFile fos=new RandomAccessFile(out, "rw");
			
			if (pos>0) {
				fos.seek(pos);
			}
			
			k=0;
			
			i=0;
			
			updateGUI("Writing "+out.getName());
			while (true) {
				
				if (i>=len) {
					break;
				}
				rgb=getRGB();
				
				writeBuffer(fos);
				buffer[k]=(byte)getRed(rgb) ;
				k++;
				i++;
				
				if (i>=len) {
					break;
				}
				writeBuffer(fos);
				buffer[k]=(byte)getGreen(rgb) ;
				k++;
				i++;
				
				if (i>=len) {
					break;
				}
				writeBuffer(fos);
				buffer[k]=(byte)getBlue(rgb) ;
				k++;
				i++;
				
				nextPixel();
				
			}
			
			if (len>=3) {
				fos.write(buffer,0,k);
			} else {
				fos.write(buffer,0,(int)len);
			}
						
			fos.close();
			updateGUI("Done writing "+out.getName());
			
		} catch (IOException e) {
			e.printStackTrace();
			updateGUI("Error: "+e.getMessage());
		}
	}
	
//	private void writeBuffer(FileOutputStream fw) throws IOException {
	private void writeBuffer(RandomAccessFile raf) throws IOException {
		if (k==buffer.length) {
			raf.write(buffer,0,k);
			k=0;
		}
	}
	
	private void updateGUI(Object arg) {
		setChanged();
		notifyObservers(arg);
	}
	
	private void nextPixel() {
		x++;
		if (x==bi.getWidth()) {
			x=0;
			y++;
		}
	}
	
	private int getInt() {
		return bi.getRGB(x, y)+(2<<23);
	}
	
	private int getRGB() {
		return bi.getRGB(x,y);
	}
	
	private int getRed(int rgb) {
		return (rgb >> 16) & 0xFF;
	}

	private int getGreen(int rgb) {
		return (rgb >> 8) & 0xFF;
	}

	private int getBlue(int rgb) {
		return (rgb >> 0) & 0xFF;
	}
	
	
	private long intToLong(int i1,int i2,int i3,int i4) {
		return (long)i4<<48|(long)i3<<32|(long)i2<<16|(long)i1;
	}
}
